﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Telerik.Windows.Controls;

namespace PageNavigation_SlideTransition
{
	public class SlideTransition : ITransition
	{
		private TimeSpan timeDuration = new TimeSpan(0, 0, 1);
		private Control newPage;
		private Control currentPage;
		private Storyboard currentAnimation;

		public SlideTransition()
		{
		}

		public SlideTransition(TimeSpan duration)
		{
			this.timeDuration = duration;
			this.IsRunning = false;
		}

		public Storyboard CurrentStoryboard
		{
			get
			{
				return this.currentAnimation;
			}
			set
			{
				this.currentAnimation = value;
			}
		}

		public bool IsRunning
		{
			get;
			set;
		}

		public void StopTransition()
		{
			if (this.CurrentStoryboard != null)
			{
				this.CurrentStoryboard.Stop();
			}
		}

		public void Begin(IFrame page)
		{
			Panel target = NavigationService.GetNavigationService().Target;
			IFrame previousFrame = null;
			if (target.Children.Count != 0)
			{
				previousFrame = target.Children[0] as IFrame;
				if (previousFrame is RadFrame)
				{
					previousFrame = null;
				}
			}

			this.IsRunning = true;
			this.newPage = page as Control;
			this.currentPage = previousFrame as Control;

			target.Children.Clear();
			target.Children.Add(this.newPage);

			if (this.currentPage != this.newPage)
			{
				if (this.currentPage != null)
				{
					this.currentPage.RenderTransform = null;

					target.Children.Add(this.currentPage);
					target.UpdateLayout();

					Storyboard sb = this.GetStoryboard(target);
					this.CurrentStoryboard = sb;
					if (this.newPage.ActualHeight == 0.0)
					{
						target.Height = this.currentPage.ActualHeight * 2;
					}
					target.Clip =
							 new RectangleGeometry()
							 {
								 Rect = new Rect(0, 0, this.currentPage.ActualWidth, this.currentPage.ActualHeight)
							 };
					sb.Completed += new EventHandler(this.Sb_Completed);
					sb.Begin();
				}
			}
			else
			{
				target.Children.Clear();
				target.Children.Add(this.newPage);
				target.UpdateLayout();
			}
			}

		private void Sb_Completed(object sender, EventArgs e)
		{
			Panel target = NavigationService.GetNavigationService().Target;
			if (target.Children.Count == 2)
			{
				target.Children.Remove(this.currentPage);
			}
			target.UpdateLayout();
			this.IsRunning = false;
		}

		private Storyboard GetStoryboard(Panel container)
		{
			double height = container.ActualHeight;
			double width = container.ActualWidth;

			TranslateTransform topTransform = null;
			TranslateTransform bottomTransform = null;

			FrameworkElement topContent = (FrameworkElement)container.Children[1];
			FrameworkElement bottomContent = (FrameworkElement)container.Children[0];

			if (topTransform == null)
			{
			    topTransform = new TranslateTransform();
			    topContent.RenderTransform = topTransform;
			}
			if (bottomTransform == null)
			{
			    bottomTransform = new TranslateTransform();
			    bottomContent.RenderTransform = bottomTransform;
			}
			Storyboard slideAnimation = new Storyboard();
			slideAnimation.Duration = this.timeDuration;
			this.SetDoubleAnimation(slideAnimation, topTransform, bottomTransform, height, width);
			return slideAnimation;
		}

		private void SetDoubleAnimation(Storyboard slideAnimation, TranslateTransform topTransform, TranslateTransform bottomTransform, double height, double width)
		{
			DoubleAnimation top = new DoubleAnimation();
			DoubleAnimation bottom = new DoubleAnimation();
			topTransform.X = -width;
			top.Duration = this.timeDuration;
			top.From = -width / 2;
			top.To = -width;
			bottom.Duration = this.timeDuration;
			bottom.From = width / 2;
			bottom.To = 0;
			slideAnimation.Children.Add(top);
			slideAnimation.Children.Add(bottom);
			Storyboard.SetTarget(top, topTransform);
			Storyboard.SetTargetProperty(top, new PropertyPath("X"));
			Storyboard.SetTarget(bottom, bottomTransform);
			Storyboard.SetTargetProperty(bottom, new PropertyPath("X"));
			}
	}
}
